@@ -240,11 +240,8 @@ shm_format(void *data, struct wl_shm *wl_shm, uint32_t format)
     struct wayland *wayl = data;
 
     switch (format) {
-    case WL_SHM_FORMAT_XRGB2101010: wayl->shm_have_xrgb2101010 = true; break;
     case WL_SHM_FORMAT_ARGB2101010: wayl->shm_have_argb2101010 = true; break;
-    case WL_SHM_FORMAT_XBGR2101010: wayl->shm_have_xbgr2101010 = true; break;
     case WL_SHM_FORMAT_ABGR2101010: wayl->shm_have_abgr2101010 = true; break;
-    case WL_SHM_FORMAT_XBGR16161616: wayl->shm_have_xbgr161616 = true; break;
     case WL_SHM_FORMAT_ABGR16161616: wayl->shm_have_abgr161616 = true; break;
     }
 
@@ -1064,6 +1061,7 @@ xdg_surface_configure(void *data, struct xdg_surface *xdg_surface,
 
     bool wasnt_configured = !win->is_configured;
     bool was_resizing = win->is_resizing;
+    bool was_fullscreen = win->is_fullscreen;
     bool csd_was_enabled = win->csd_mode == CSD_YES && !win->is_fullscreen;
     int new_width = win->configure.width;
     int new_height = win->configure.height;
@@ -1134,11 +1132,26 @@ xdg_surface_configure(void *data, struct xdg_surface *xdg_surface,
     else
         term_visual_focus_out(term);
 
-    if (!resized && !term->render.pending.grid) {
+    /*
+     * Update opaque region if fullscreen state changed, also need to
+     * render, since we use different buffer types with and without
+     * alpha
+     */
+    if (was_fullscreen != win->is_fullscreen) {
+        wayl_win_alpha_changed(win);
+        render_refresh(term);
+    }
+
+    const bool will_render_soon = resized ||
+                                  term->render.refresh.grid ||
+                                  term->render.pending.grid;
+
+    if (!will_render_soon) {
         /*
-         * If we didn't resize, we won't be committing a new surface
-         * anytime soon. Some compositors require a commit in
-         * combination with an ack - make them happy.
+         * If we didn't resize, and aren't refreshing for other
+         * reasons, we won't be committing a new surface anytime
+         * soon. Some compositors require a commit in combination with
+         * an ack - make them happy.
          */
         wl_surface_commit(win->surface.surf);
     }
@@ -1180,6 +1193,27 @@ static const struct zxdg_toplevel_decoration_v1_listener xdg_toplevel_decoration
     .configure = &xdg_toplevel_decoration_configure,
 };
 
+#if defined(HAVE_EXT_BACKGROUND_EFFECT)
+static void
+ext_background_capabilities(
+    void *data,
+    struct ext_background_effect_manager_v1 *ext_background_effect_manager_v1,
+    uint32_t flags)
+{
+    struct wayland *wayl = data;
+
+    wayl->have_background_blur =
+        !!(flags & EXT_BACKGROUND_EFFECT_MANAGER_V1_CAPABILITY_BLUR);
+
+    LOG_DBG("compositor supports background blur: %s",
+            wayl->have_background_blur ? "yes" : "no");
+}
+
+static const struct ext_background_effect_manager_v1_listener background_manager_listener = {
+    .capabilities = &ext_background_capabilities,
+};
+#endif  /* HAVE_EXT_BACKGROUND_EFFECT */
+
 static bool
 fdm_repeat(struct fdm *fdm, int fd, int events, void *data)
 {
@@ -1532,6 +1566,31 @@ handle_global(void *data, struct wl_registry *registry,
             wayl->color_management.manager, &color_manager_listener, wayl);
     }
 
+#if defined(HAVE_XDG_TOPLEVEL_TAG)
+    else if (streq(interface, xdg_toplevel_tag_manager_v1_interface.name)) {
+        const uint32_t required = 1;
+        if (!verify_iface_version(interface, version, required))
+            return;
+
+        wayl->toplevel_tag_manager = wl_registry_bind(
+            wayl->registry, name, &xdg_toplevel_tag_manager_v1_interface, required);
+    }
+#endif
+#if defined(HAVE_EXT_BACKGROUND_EFFECT)
+    else if (streq(interface, ext_background_effect_manager_v1_interface.name)) {
+        const uint32_t required = 1;
+        if (!verify_iface_version(interface, version, required))
+            return;
+
+        wayl->background_effect_manager = wl_registry_bind(
+            wayl->registry, name,
+            &ext_background_effect_manager_v1_interface, required);
+
+        ext_background_effect_manager_v1_add_listener(
+            wayl->background_effect_manager, &background_manager_listener, wayl);
+    }
+#endif
+
 #if defined(FOOT_IME_ENABLED) && FOOT_IME_ENABLED
     else if (streq(interface, zwp_text_input_manager_v3_interface.name)) {
         const uint32_t required = 1;
@@ -1545,6 +1604,7 @@ handle_global(void *data, struct wl_registry *registry,
             seat_add_text_input(&it->item);
     }
 #endif
+
 }
 
 static void
@@ -1775,7 +1835,7 @@ wayl_init(struct fdm *fdm, struct key_binding_manager *key_binding_manager,
     }
 
     if (wayl->toplevel_icon_manager == NULL) {
-        LOG_WARN("compositor does not implement the XDG toplevel icon protocol");
+        LOG_WARN("compositor does not implement the xdg-toplevel-icon protocol");
     }
 
 #if defined(FOOT_IME_ENABLED) && FOOT_IME_ENABLED
@@ -1854,6 +1914,15 @@ wayl_destroy(struct wayland *wayl)
         zwp_text_input_manager_v3_destroy(wayl->text_input_manager);
 #endif
 
+#if defined(HAVE_XDG_TOPLEVEL_TAG)
+    if (wayl->toplevel_tag_manager != NULL)
+        xdg_toplevel_tag_manager_v1_destroy(wayl->toplevel_tag_manager);
+#endif
+#if defined(HAVE_EXT_BACKGROUND_EFFECT)
+    if (wayl->background_effect_manager != NULL)
+        ext_background_effect_manager_v1_destroy(wayl->background_effect_manager);
+#endif
+
     if (wayl->color_management.img_description != NULL)
         wp_image_description_v1_destroy(wayl->color_management.img_description);
     if (wayl->color_management.manager != NULL)
@@ -1957,8 +2026,6 @@ wayl_win_init(struct terminal *term, const char *token)
         goto out;
     }
 
-    wayl_win_alpha_changed(win);
-
     wl_surface_add_listener(win->surface.surf, &surface_listener, win);
 
     if (wayl->fractional_scale_manager != NULL && wayl->viewporter != NULL) {
@@ -1971,6 +2038,16 @@ wayl_win_init(struct terminal *term, const char *token)
             win->fractional_scale, &fractional_scale_listener, win);
     }
 
+#if defined(HAVE_EXT_BACKGROUND_EFFECT)
+    if (wayl->background_effect_manager != NULL) {
+        win->surface.background_effect =
+            ext_background_effect_manager_v1_get_background_effect(
+                wayl->background_effect_manager, win->surface.surf);
+    }
+#endif
+
+    wayl_win_alpha_changed(win);
+
     win->xdg_surface = xdg_wm_base_get_xdg_surface(wayl->shell, win->surface.surf);
     xdg_surface_add_listener(win->xdg_surface, &xdg_surface_listener, win);
 
@@ -1979,6 +2056,21 @@ wayl_win_init(struct terminal *term, const char *token)
 
     xdg_toplevel_set_app_id(win->xdg_toplevel, conf->app_id);
 
+#if defined(HAVE_XDG_TOPLEVEL_TAG)
+    if (conf->toplevel_tag != NULL && conf->toplevel_tag[0] != '\0') {
+        if (wayl->toplevel_tag_manager != NULL) {
+            xdg_toplevel_tag_manager_v1_set_toplevel_tag(
+                wayl->toplevel_tag_manager, win->xdg_toplevel, conf->toplevel_tag);
+
+            /* TODO: the description is recommended to be the tag, but translated */
+            xdg_toplevel_tag_manager_v1_set_toplevel_description(
+                wayl->toplevel_tag_manager, win->xdg_toplevel, conf->toplevel_tag);
+        } else {
+            LOG_WARN("compositor does not implement the xdg-toplevel-tag protocol");
+        }
+    }
+#endif
+
     if (wayl->toplevel_icon_manager != NULL) {
         const char *app_id =
             term->app_id != NULL ? term->app_id : term->conf->app_id;
@@ -2085,6 +2177,8 @@ wayl_win_destroy(struct wl_window *win)
 
     struct terminal *term = win->term;
 
+    render_wait_for_preapply_damage(term);
+
     if (win->csd.move_timeout_fd != -1)
         close(win->csd.move_timeout_fd);
 
@@ -2160,7 +2254,12 @@ wayl_win_destroy(struct wl_window *win)
         free(it->item);
 
         tll_remove(win->xdg_tokens, it);
-}
+    }
+
+#if defined(HAVE_EXT_BACKGROUND_EFFECT)
+    if (win->surface.background_effect != NULL)
+        ext_background_effect_surface_v1_destroy(win->surface.background_effect);
+#endif
 
     if (win->surface.color_management != NULL)
         wp_color_management_surface_v1_destroy(win->surface.color_management);
@@ -2400,10 +2499,16 @@ void
 wayl_win_alpha_changed(struct wl_window *win)
 {
     struct terminal *term = win->term;
+    struct wayland *wayl = term->wl;
 
-    if (term->colors.alpha == 0xffff) {
-        struct wl_region *region = wl_compositor_create_region(
-            term->wl->compositor);
+    /*
+     * When fullscreened, transparency is disabled (see render.c).
+     * Update the opaque region to match.
+     */
+    const bool is_opaque = term->colors.alpha == 0xffff || win->is_fullscreen;
+
+    if (is_opaque) {
+        struct wl_region *region = wl_compositor_create_region(wayl->compositor);
 
         if (region != NULL) {
             wl_region_add(region, 0, 0, INT32_MAX, INT32_MAX);
@@ -2412,6 +2517,38 @@ wayl_win_alpha_changed(struct wl_window *win)
         }
     } else
         wl_surface_set_opaque_region(win->surface.surf, NULL);
+
+#if defined(HAVE_EXT_BACKGROUND_EFFECT)
+    if (term_theme_get(term)->blur) {
+        if (wayl->have_background_blur) {
+            xassert(win->surface.background_effect != NULL);
+
+            if (is_opaque) {
+                /* No transparency, disable blur */
+                LOG_DBG("disabling background blur");
+                ext_background_effect_surface_v1_set_blur_region(
+                    win->surface.background_effect, NULL);
+            } else {
+                /* We have transparency, enable blur if user has enabled it */
+                struct wl_region *region = wl_compositor_create_region(wayl->compositor);
+                if (region != NULL) {
+                    LOG_DBG("enabling background blur");
+
+                    wl_region_add(region, 0, 0, INT32_MAX, INT32_MAX);
+                    ext_background_effect_surface_v1_set_blur_region(
+                        win->surface.background_effect, region);
+                    wl_region_destroy(region);
+                }
+            }
+        } else {
+            static bool have_warned = false;
+            if (!have_warned) {
+                LOG_WARN("background blur requested, but compositor does not support it");
+                have_warned = true;
+            }
+        }
+    }
+#endif /* HAVE_EXT_BACKGROUND_EFFECT */
 }
 
 static void
